import dataclasses
import pathlib
import torch
import irisml.core


class Task(irisml.core.TaskBase):
    """Save the model's state_dict to the specified file."""
    VERSION = '0.1.0'
    CACHE_ENABLED = False

    @dataclasses.dataclass
    class Inputs:
        model: torch.nn.Module

    @dataclasses.dataclass
    class Config:
        path: pathlib.Path

    def execute(self, inputs):
        state_dict = inputs.model.state_dict()
        torch.save(state_dict, self._config.path)
        return self.Outputs()



import dataclasses
import torch
import irisml.core


class Task(irisml.core.TaskBase):
    """Get the largest Topk values and indices.

    Inputs:
        tensor (torch.Tensor): The input tensor

    Config:
        k (int): The "k".
    """
    VERSION = '0.1.0'

    @dataclasses.dataclass
    class Inputs:
        tensor: torch.Tensor

    @dataclasses.dataclass
    class Config:
        k: int

    @dataclasses.dataclass
    class Outputs:
        values: torch.Tensor
        indices: torch.LongTensor

    def execute(self, inputs):
        result = torch.topk(inputs.tensor, self.config.k)
        return self.Outputs(result.values, result.indices)

    def dry_run(self, inputs):
        shape = [*inputs.tensor.shape[:-1], self.config.k]
        fake_values = torch.zeros(shape)
        fake_indices = torch.arange(self.config.k).expand(shape)
        return self.Outputs(fake_values, fake_indices)



import dataclasses
import random
import torch.utils.data
import irisml.core


class Task(irisml.core.TaskBase):
    """Get a subset of the given dataset."""
    VERSION = '0.1.0'

    @dataclasses.dataclass
    class Config:
        num_images: int

    @dataclasses.dataclass
    class Inputs:
        dataset: torch.utils.data.Dataset

    @dataclasses.dataclass
    class Outputs:
        dataset: torch.utils.data.Dataset = None

    def execute(self, inputs):
        num_images = len(inputs.dataset)
        new_num_images = self.config.num_images
        indexes = random.sample(range(num_images), new_num_images)
        new_dataset = torch.utils.data.Subset(inputs.dataset, indexes)
        return self.Outputs(new_dataset)



import dataclasses
import logging
import random
import typing
import irisml.core
import torch
import PIL.Image
import PIL.ImageDraw

logger = logging.getLogger(__name__)


class Task(irisml.core.TaskBase):
    """Generate a fake object detection dataset.

    A generated dataset returns (image:PIL.Image, targets:List[Tensor[N, 5]])
    """
    VERSION = '0.1.0'

    @dataclasses.dataclass
    class Config:
        num_images: int = 100
        num_classes: int = 10
        num_max_boxes: int = 10  # The max number of objects per image.

    @dataclasses.dataclass
    class Outputs:
        dataset: torch.utils.data.Dataset = None
        num_classes: int = 0

    _IMAGE_SIZE = (320, 320)  # Width, height of an image

    class FakeDataset(torch.utils.data.Dataset):
        def __init__(self, targets: typing.List[torch.Tensor], colors: typing.List[typing.Tuple[int, int, int]]):
            self._targets = targets
            self._colors = colors

        def __len__(self):
            return len(self._targets)

        def __getitem__(self, index):
            targets = self._targets[index]
            return self._generate_image(targets), targets

        def _generate_image(self, targets):
            image = PIL.Image.new('RGB', Task._IMAGE_SIZE)
            draw = PIL.ImageDraw.Draw(image)
            for t in targets:
                rect = [t[1] * Task._IMAGE_SIZE[0], t[2] * Task._IMAGE_SIZE[1], t[3] * Task._IMAGE_SIZE[0], t[4] * Task._IMAGE_SIZE[1]]
                draw.rectangle(rect, fill=self._colors[int(t[0])])
            return image

    def execute(self, inputs):
        def _random_box():
            cx = random.random()
            cy = random.random()
            w = random.uniform(0, min(1 - cx, cx))
            h = random.uniform(0, min(1 - cy, cy))
            return [cx - w / 2, cy - h / 2, cx + w / 2, cy + h / 2]

        targets = [torch.tensor([[random.randrange(self.config.num_classes), *_random_box()] for _ in range(random.randrange(self.config.num_max_boxes + 1))]).reshape(-1, 5)
                   for _ in range(self.config.num_images)]

        colors = [(random.randint(0, 255), random.randint(0, 255), random.randint(0, 255)) for i in range(self.config.num_classes)]
        logger.debug(f"Generated labels: {colors}")
        dataset = Task.FakeDataset(targets, colors)
        return self.Outputs(dataset, num_classes=self.config.num_classes)



import dataclasses
import logging
from typing import Optional
import torch.utils.data
import irisml.core


class Task(irisml.core.TaskBase):
    """Get a train/val split of a dataset.

    If val_dataset is not provided, then the train_dataset will be split into a train and val dataset. E.g. if train_val_split is 0.8, then 80% of the
    train_dataset will be used for training and 20% will be used for validation.

    If val_dataset is provided, then it will be used as the validation dataset and the train_dataset will be used for training as-is.
    """
    VERSION = '0.1.0'

    @dataclasses.dataclass
    class Config:
        train_val_split: float

    @dataclasses.dataclass
    class Inputs:
        train_dataset: torch.utils.data.Dataset
        val_dataset: Optional[torch.utils.data.Dataset] = None

    @dataclasses.dataclass
    class Outputs:
        train_dataset: torch.utils.data.Dataset
        val_dataset: torch.utils.data.Dataset

    def execute(self, inputs):
        if self.config.train_val_split < 0 or self.config.train_val_split > 1:
            raise ValueError("train_val_split must be between 0 and 1")

        if not inputs.val_dataset:
            num_images = len(inputs.train_dataset)
            num_train_images = int(num_images * self.config.train_val_split)
            num_val_images = num_images - num_train_images
            train_dataset, val_dataset = torch.utils.data.random_split(inputs.train_dataset, [num_train_images, num_val_images])
            logging.info(f"Split train dataset into {len(train_dataset)} train images and {len(val_dataset)} val images")
        else:
            train_dataset, val_dataset = inputs.train_dataset, inputs.val_dataset
            logging.info(f"Skip splitting - val dataset is already provided. Using {len(train_dataset)} train images and {len(val_dataset)} val images")

        return self.Outputs(train_dataset, val_dataset)

    def dry_run(self, inputs):
        return self.execute(inputs)



import unittest
from irisml.tasks.get_fake_object_detection_dataset import Task


class TestGetFakeObjectDetectionDataset(unittest.TestCase):
    def test_simple(self):
        config = Task.Config(num_images=100, num_classes=10, num_max_boxes=10)
        task = Task(config)
        outputs = task.execute(None)
        dataset = outputs.dataset
        self.assertEqual(len(dataset), 100)
        class_set = set(int(t[0]) for x in dataset for t in x[1])
        self.assertEqual(len(class_set), 10)
        self.assertTrue(all(len(x[1]) <= 10 for x in dataset))
        self.assertLess(sum(len(x[1]) for x in dataset), 1000)
        self.assertTrue(all(len(x[1].shape) == 2 for x in dataset))
        self.assertTrue(all(0 <= x[1] < x[3] <= 1 for _, t in dataset for x in t))
        self.assertTrue(all(0 <= x[2] < x[4] <= 1 for _, t in dataset for x in t))



import unittest
import torch
from irisml.tasks.get_dataset_stats import Task


class TestGetDatasetStats(unittest.TestCase):
    def test_multiclass_classification(self):
        class Dataset(torch.utils.data.Dataset):
            def __len__(self):
                return 3

            def __getitem__(self, index):
                return (torch.zeros((3, 100, 100)), torch.Tensor([index]))

            def __iter__(self):
                yield self[0]
                yield self[1]
                yield self[2]

        dataset = Dataset()

        task = Task(Task.Config())
        outputs = task.execute(Task.Inputs(dataset))
        self.assertEqual(outputs.num_images, 3)
        self.assertEqual(outputs.num_classes, 3)
        self.assertEqual(outputs.dataset_type, 'multiclass_classification')

    def test_multilabel_classification(self):
        class Dataset(torch.utils.data.Dataset):
            DATA = [torch.Tensor([0, 1, 2]), torch.Tensor([0]), torch.Tensor([])]

            def __len__(self):
                return len(self.DATA)

            def __getitem__(self, index):
                return (torch.zeros((3, 100, 100)), self.DATA[index])

            def __iter__(self):
                yield self[0]
                yield self[0]
                yield self[0]

        dataset = Dataset()

        task = Task(Task.Config())
        outputs = task.execute(Task.Inputs(dataset))
        self.assertEqual(outputs.num_images, 3)
        self.assertEqual(outputs.num_classes, 3)
        self.assertEqual(outputs.dataset_type, 'multilabel_classification')

    def test_object_detection(self):
        class Dataset(torch.utils.data.Dataset):
            DATA = [torch.Tensor([[0, 0.1, 0.1, 0.2, 0.2], [1, 0.1, 0.1, 0.2, 0.2], [2, 0.1, 0.1, 0.2, 0.2]]), torch.Tensor([[0, 0.1, 0.1, 0.2, 0.2]]), torch.Tensor([])]

            def __len__(self):
                return len(self.DATA)

            def __getitem__(self, index):
                return (torch.zeros((3, 100, 100)), self.DATA[index])

            def __iter__(self):
                yield self[0]
                yield self[0]
                yield self[0]

        dataset = Dataset()

        task = Task(Task.Config())
        outputs = task.execute(Task.Inputs(dataset))
        self.assertEqual(outputs.num_images, 3)
        self.assertEqual(outputs.num_classes, 3)
        self.assertEqual(outputs.dataset_type, 'object_detection')



import unittest
from irisml.core import Context
from irisml.tasks.get_fake_image_classification_dataset import Task


class TestGetFakeImageClassificationDataset(unittest.TestCase):
    def test_simple(self):
        config = Task.Config(num_images=100, num_classes=10)
        task = Task(config, Context())
        outputs = task.execute(None)
        dataset = outputs.dataset
        self.assertEqual(len(dataset), 100)
        class_set = set(x[1] for x in dataset)
        self.assertEqual(len(class_set), 10)



import dataclasses
import torch
import irisml.core


class Task(irisml.core.TaskBase):
    """Calculate cosine similarity between two sets of vectors.

    Inputs:
        tensor0 (torch.Tensor): Shape [N0, M]
        tensor1 (torch.Tensor): Shape [N1, M]

    Outputs:
        cosine_similarity (torch.Tensor): Shape [N0, N1]
    """
    VERSION = '0.1.0'

    @dataclasses.dataclass
    class Inputs:
        tensor0: torch.Tensor
        tensor1: torch.Tensor

    @dataclasses.dataclass
    class Outputs:
        cosine_similarity: torch.Tensor

    def execute(self, inputs):
        if inputs.tensor0.shape[1] != inputs.tensor1.shape[1] or len(inputs.tensor0.shape) != 2 or len(inputs.tensor1.shape) != 2:
            raise RuntimeError(f"Input tensors have unexpected shape: tensor0.shape={inputs.tensor0.shape}, tensor1.shape={inputs.tensor1.shape}")

        eps = 1e-8
        norm0 = inputs.tensor0.norm(dim=1)[:, None]
        norm1 = inputs.tensor1.norm(dim=1)[:, None]
        normalized_tensor0 = inputs.tensor0 / torch.clamp(norm0, min=eps)
        normalized_tensor1 = inputs.tensor1 / torch.clamp(norm1, min=eps)
        cosine_similarity = torch.mm(normalized_tensor0, normalized_tensor1.transpose(0, 1))
        assert cosine_similarity.shape == (inputs.tensor0.shape[0], inputs.tensor1.shape[0])
        return self.Outputs(cosine_similarity)

    def dry_run(self, inputs):
        fake_result = torch.zeros((inputs.tensor0.shape[0], inputs.tensor1.shape[0]))
        return self.Outputs(fake_result)
